#ifndef DateiDialog_H
#define DateiDialog_H

#include "Array.h"
#include "Thread.h"

namespace Framework
{
    class Text; // Text.h

    void InitDialog();

    // Erstellt einen Datei ffnen/speichern Dialog
    class DateiDialog
    {
    private:
        RCArray< Text > *typeName;
        RCArray< Text > *type;
        int fileIndex;
        int ref;

    public:
        // Konstruktor
        __declspec( dllexport ) DateiDialog();
        // Destruktor
        __declspec( dllexport ) ~DateiDialog();
        // Lscht die Liste mit zugelassenen Dateitypen
        __declspec( dllexport ) void removeDateiTypen();
        // Fgt einen zugelassenen Dateityp hinzu
        //  name: Der Name des Dateitypes. Ist fr den Nutzer in der Select Box sichtbar
        //  typ: Der Dateityp, der ausgewhlt werden darf
        __declspec( dllexport ) void addDateiTyp( char *name, char *typ );
        // Fgt einen zugelassenen Dateityp hinzu
        //  name: Der Name des Dateitypes. Ist fr den Nutzer in der Select Box sichtbar
        //  typ: Der Dateityp, der ausgewhlt werden darf
        __declspec( dllexport ) void addDateiTyp( Text *name, Text *typ );
        // Setzt den zu Beginn ausgewhlten Dateityp
        //  i: Der Index des Dateityps. Der, der als erstes hinzugefgt wurde, hat den Index 0.
        __declspec( dllexport ) void setDateiTypAuswahl( int i );
        // Zeigt den Dateidialog an
        //  open: true, wenn der Dialog zum ffnen dienen soll. false zum Speichern
        //  return: Den Pfad zur ausgewhlten Datei
        __declspec( dllexport ) Text *anzeigen( bool open ) const;
        // Erhht den Reference Counting Zhler.
        //  return: this.
        __declspec( dllexport ) DateiDialog *getThis();
        // Verringert den Reference Counting Zhler. Wenn der Zhler 0 erreicht, wird das Zeichnung automatisch gelscht.
        //  return: 0.
        __declspec( dllexport ) DateiDialog *release();
    };

    // Verwaltet einen Datei ffnen/speichern Dialog ohne zu warten
    class DateiDialogTh : public Thread
    {
    private:
        DateiDialog *dialog;
        Text *ret;
        bool open;

    public:
        // Konstruktor
        __declspec( dllexport ) DateiDialogTh();
        // Destruktor
        __declspec( dllexport ) ~DateiDialogTh();
        // Legt fest, ob der Dialog zum ffnen oder speichern ist
        //  b: 1, wenn er zum fnen ist. 0, wenn er zum speichern ist
        __declspec( dllexport ) void setOpen( bool b );
        // Lscht die liste mit erlaubten Dateitypen
        __declspec( dllexport ) void removeDateiTypen();
        // Fgt einen zugelassenen Dateityp hinzu
        //  name: Der Name des Dateitypes. Ist fr den Nutzer in der Select Box sichtbar
        //  typ: Der Dateityp, der ausgewhlt werden darf
        __declspec( dllexport ) void addDateiTyp( char *name, char *typ );
        // Fgt einen zugelassenen Dateityp hinzu
        //  name: Der Name des Dateitypes. Ist fr den Nutzer in der Select Box sichtbar
        //  typ: Der Dateityp, der ausgewhlt werden darf
        __declspec( dllexport ) void addDateiTyp( Text *name, Text *typ );
        // Setzt den zu Beginn ausgewhlten Dateityp
        //  i: Der Index des Dateityps. Der, der als erstes hinzugefgt wurde, hat den Index 0.
        __declspec( dllexport ) void setDateiTypAuswahl( int i );
        // Diese Funktion wird von der Klasse selbst aufgerufen.
        // Benutze die start Funktion um den Dialog anzuzeigen
        __declspec( dllexport ) void thread() override;
        // Gibt den Pfad zur Datei zurck.
        // Funktioniert erst, nachdem der Thread beendet wurde
        __declspec( dllexport ) Text *getPfad() const;
        // Gibt den Pfad zur Datei ohne erhhten Reference Counter zurck.
        // Funktioniert erst, nachdem der Thread beendet wurde
        __declspec( dllexport ) Text *zPfad() const;
    };
};

#endif